#include "Eventfulness.h"

#include "World.h"
#include "CreateBar.h"
#include "NotificationBar.h"
#include "WorldWindow.h"
#include "MashWindow.h"
#include "RogueWindow.h"
#include "Render.h"
#include "Stat.h"
#include "GameOver.h"

#include "Note.h"

#include <iostream>
#include <sstream>
#include <string>
#include <math.h>

void Eventfulness::OnCreateBarButtonClick(BarButton* bbutton)
{		
	if (bbutton->m_NoteString != "") // Notification
	{
		if (bbutton->m_Note == NULL || !bbutton->m_Note->Visible())
		{
			Note* notewindow = new Note();
			notewindow->Init(bbutton->m_NoteString);
			notewindow->SetPosition( bbutton->m_Button->GetAbsolutePosition() + sf::Vector2f(64,0));
			m_Render->Register(notewindow->GetPointer());
			bbutton->m_Note = notewindow;
		}
		else
		{
			bbutton->m_Note->OnClick();
			m_NotificationBar->RemoveNotification(bbutton->m_Index-100);
		}
	}
	else if (bbutton->m_Obj->m_Rooms[0]->Type == NEW)
	{
		NewButtonPoked(bbutton);
	}
	else // spawn
	{
		Object* newObj = bbutton->m_Obj->Spawn();
		CreateNewObject(newObj,bbutton->m_Obj->m_Name);
	}
}

void Eventfulness::CreateNewObject(Object* newObj,std::string type)
{
	m_World->AddObject(newObj);
	
	std::stringstream nshort;
	std::stringstream nlong;
	
	nshort << "c";
	nshort << type;
	
	nlong << "New object created: ";
	nlong << newObj->m_Name;
	nlong << ".";
	
	m_NotificationBar->NewMessage(nshort.str(),nlong.str());
	
	m_World->GetHumanPlayer()->m_Risk += 10*pow(newObj->m_Rooms.size(),2);
	m_World->Turn();
	m_WorldWindow->Update( m_World->GetVisibles(0) );
	m_Stat->Update();
	m_MashWindow->AddText();
}

void Eventfulness::OnCreateBarButtonRightClick(BarButton* bbutton)
{	
	if (bbutton->m_NoteString != "")
	{
		m_NotificationBar->RemoveNotification(bbutton->m_Index-100);
	}
	else if (bbutton->m_Obj->m_Rooms[0]->Type != NEW)
	{
		m_CreateBar->RemoveButton(bbutton->m_Index);
	}
}

void Eventfulness::SetWorld(World* w)
{
	m_World = w;
}

void Eventfulness::SetStat(Stat* s)
{
	m_Stat = s;
}


void Eventfulness::SetNotificationBar(NotificationBar* nbar)
{
	m_NotificationBar = nbar;
}

void Eventfulness::SetCreateBar(CreateBar* cbar)
{
	m_CreateBar = cbar;
}

void Eventfulness::SetWorldWindow(WorldWindow* ww)
{
	m_WorldWindow = ww;
}

void Eventfulness::SetMashWindow(MashWindow* mw)
{
	m_MashWindow = mw;
}

void Eventfulness::SetRogueWindow(RogueWindow* rw)
{
	m_RogueWindow = rw;
}


void Eventfulness::SetRender(Render* rend)
{
	m_Render = rend;
}

void Eventfulness::KeyPressed(sf::Event::KeyEvent keyEvent)
{	
	if (m_MashWindow->m_Active)
	{
		if (keyEvent.code == sf::Keyboard::Space)
		{
			m_MashWindow->SetActive(false);
		}
		else
		{
			m_World->Turn();
			m_WorldWindow->Update( m_World->GetVisibles(0) );
			m_Stat->Update();
			m_MashWindow->AddText();
		}
	}
	else
	{
		if (keyEvent.code == sf::Keyboard::Space)
		{
			m_MashWindow->SetActive(true);
		}
		if (keyEvent.code == -1)	m_NotificationBar->PokeNotification(0);
		
		if (keyEvent.code == sf::Keyboard::Num1)	m_CreateBar->PokeButton(0);
		if (keyEvent.code == sf::Keyboard::Num2)	m_CreateBar->PokeButton(1);
		if (keyEvent.code == sf::Keyboard::Num3)	m_CreateBar->PokeButton(2);
		if (keyEvent.code == sf::Keyboard::Num4)	m_CreateBar->PokeButton(3);
		if (keyEvent.code == sf::Keyboard::Num5)	m_CreateBar->PokeButton(4);
		if (keyEvent.code == sf::Keyboard::Num6)	m_CreateBar->PokeButton(5);
		if (keyEvent.code == sf::Keyboard::Num7)	m_CreateBar->PokeButton(6);
		if (keyEvent.code == sf::Keyboard::Num8)	m_CreateBar->PokeButton(7);
		if (keyEvent.code == sf::Keyboard::Num9)	m_CreateBar->PokeButton(8);
		if (keyEvent.code == sf::Keyboard::Num0)	m_CreateBar->PokeButton(9);
		
		
		
	}
	
}

void Eventfulness::NewMessage(std::string smessage, std::string lmessage)
{
	m_NotificationBar->NewMessage(smessage,lmessage);
}

void Eventfulness::NewButtonPoked(BarButton* bb)
{
	m_RogueWindow->m_TargetButton = bb;
	m_RogueWindow->Show(true);
	BringToFront(m_RogueWindow->GetPointer());
}

void Eventfulness::BringToFront(sfg::Window::Ptr window)
{
	m_Render->BringToFront(window);
}

void Eventfulness::ItsGameOver(bool win)
{
	GameOver* gameOver = new GameOver();
	gameOver->Init(win);
	m_Render->Register(gameOver->GetPointer());
}