#include "Glob.h"
#include "RogueWindow.h"
#include "CreateBar.h"
#include "Object.h"
#include "BarButton.h"

#include <iostream>
#include <sstream>
#include <algorithm>

#define WIDTH 46
#define HEIGHT 18

extern Glob* glob;

int cord(int y, int x)
{
	return y*HEIGHT + x;
}
/*
std::pair<int,int> dorc (int in)
{	
	std::pair<int,int> out;
	
	out[0] = in % HEIGHT;
	out[1] = in-out[0] / HEIGHT;
}
*/

sfg::Window::Ptr RogueWindow::GetPointer()
{
	return m_Window;
}

int RogueWindow::DRand(int in, int shuffle)
{
	return m_shuffledints[ (in+shuffle)%m_shuffledints.size() ] % in;
}

bool RogueWindow::Init(CreateBar* cbar)
{
	m_CreateBar = cbar;
	m_CurrentRoomType = PLAYERBASE;
	m_TargetButton = NULL;
	
	for (int i=0;i<40;i++)
	{
		m_shuffledints.push_back(i);
	}
	std::random_shuffle ( m_shuffledints.begin(), m_shuffledints.end() );
	
	m_Window = sfg::Window::Create(3);
 	m_Window->SetTitle(" ");
	m_Window->SetPosition( sf::Vector2f(364+40,340));
	m_Window->SetClass( "rogue" );
	
	m_Contains = sfg::Box::Create( sfg::Box::VERTICAL );
	m_FirstRow = sfg::Box::Create(sfg::Box::HORIZONTAL);
	m_SecondRow = sfg::Fixed::Create();
	
	m_RoomChooseBar = sfg::Box::Create( sfg::Box::VERTICAL );
	m_Dungeon = sfg::Table::Create();
	
	m_FirstRow->Pack(m_RoomChooseBar);
	m_FirstRow->Pack(m_Dungeon);
	
	//m_FirstRow->Put(m_RoomChooseBar, sf::Vector2f(4,0));
	//m_FirstRow->Put(m_Dungeon, sf::Vector2f(124,0));
	
	m_Description = sfg::Label::Create("");
	m_bInventory = sfg::Button::Create( "Inventory" );
	m_bDelete = sfg::Button::Create( "Delete" );
	m_bSpawn = sfg::Button::Create( "Spawn" );
	
	m_bInventory->GetSignal( sfg::Widget::OnLeftClick ).Connect( &RogueWindow::Inventory, this );
	m_bDelete->GetSignal( sfg::Widget::OnLeftClick ).Connect( &RogueWindow::Delete, this );
	m_bSpawn->GetSignal( sfg::Widget::OnLeftClick ).Connect( &RogueWindow::Spawn, this );
	
	m_Description->SetClass("rogue");
	m_bInventory->SetClass("rogue");
	m_bDelete->SetClass("rogue");
	m_bSpawn->SetClass("rogue");
	
	m_Description->SetRequisition( sf::Vector2f(138,0) );
	m_bInventory->SetRequisition( sf::Vector2f(138,40) );
	m_bDelete->SetRequisition( sf::Vector2f(138,40) );
	m_bSpawn->SetRequisition( sf::Vector2f(138,40) );
	
	m_SecondRow->Put(m_Description, sf::Vector2f( 20,10 ));
	m_SecondRow->Put(m_bInventory, sf::Vector2f( 287,0 ));
	m_SecondRow->Put(m_bDelete, sf::Vector2f( 433,0 ));
	m_SecondRow->Put(m_bSpawn, sf::Vector2f( 579,0 ));
	
	m_Contains->Pack(m_FirstRow);
	m_Contains->Pack(m_SecondRow);

	m_Window->Add(m_Contains);
	
	RefreshAvailableRooms();
	InitDungeon();
	
	//DungeonRoom* dr = new DungeonRoom();
	//dr->type = SNOOP;
	//dr->space = sf::IntRect(1,1,5,5);
	//m_DungeonRooms.push_back(dr);
	
	RefreshDungeon();
	
	Show(false);
	
	return true;
}

void RogueWindow::RefreshAvailableRooms()
{
	for (int i=0; i< m_ChooseButtons.size();i++)
	{
		delete m_ChooseButtons[i];
	}
	m_ChooseButtons.clear();
	m_RoomChooseBar->RemoveAll();
	
	std::vector<RoomType> roomtypes = glob->world->GetHumanPlayer()->m_DiscoveredRoomTypes;
	
	for (int i=0; i< roomtypes.size();i++)
	{
		RogueButton* rbutton = new RogueButton();
		rbutton->Init(this,RoomTypeToChar(roomtypes[i]));
		rbutton->m_choose = true;
		
		m_RoomChooseBar->Pack(rbutton->m_Button);
		
		m_ChooseButtons.push_back(rbutton);
	}
	
}

void RogueWindow::InitDungeon()
{
	
	for (int i=0;i<46;i++)
	{
		for (int j=0;j<18;j++)
		{
			RogueButton* rbutton = new RogueButton();
			rbutton->Init(this,' ');
			rbutton->m_choose = false;
			rbutton->m_pos = sf::Vector2i(i,j);
			
			m_Dungeon->Attach( rbutton->m_Button, sf::Rect<sf::Uint32>( i, j, 1, 1 ), sfg::Table::FILL, sfg::Table::FILL, sf::Vector2f( 0.f, 0.f ) );
			
			m_RogueButtons.push_back(rbutton);
		}
	}
}

void RogueWindow::OnButtonClick(RogueButton* rbutton)
{
	if (rbutton->m_choose)
	{
		m_CurrentRoomType = CharToRoomType(rbutton->m_ButtonChar);
	}
	else {
		AddRoomAt(rbutton->m_pos);
	}
}

void RogueWindow::OnButtonRightClick(RogueButton* rbutton)
{
	if (!rbutton->m_choose)
	{
		RemoveRoomAt(rbutton->m_pos);
	}

}


void RogueWindow::AddRoomAt( sf::Vector2i at)
{
	int numOfAts = 0;
	for (int i=0;i<m_DungeonRooms.size();i++)
	{
		if (m_DungeonRooms[i]->type == PLAYERBASE)
		{
			numOfAts++;
		}
	}
	
	if      (numOfAts == 0 && m_CurrentRoomType != PLAYERBASE)	return;
	else if (numOfAts != 0 && m_CurrentRoomType == PLAYERBASE)	return;
	
	//TODO check that there's an @, and only one @
	
	sf::Vector2i size;
	if 			(m_CurrentRoomType == ROCK) 		size = sf::Vector2i(12,7);
	else if 	(m_CurrentRoomType == BPAPER)		size = sf::Vector2i(12,7);
	else if 	(m_CurrentRoomType == XSCISSORS)	size = sf::Vector2i(12,7);
	else if 	(m_CurrentRoomType == TEMPLATE)		size = sf::Vector2i(14,9);
	else if 	(m_CurrentRoomType == CREATE)		size = sf::Vector2i(14,9);
	else if 	(m_CurrentRoomType == PING)			size = sf::Vector2i(9,5);
	else if 	(m_CurrentRoomType == PINGFAKE)		size = sf::Vector2i(9,5);
	else 											size = sf::Vector2i(5,5);
	
	
	for (int i=0;i<m_DungeonRooms.size();i++)
	{
		if (m_DungeonRooms[i]->space.intersects(sf::IntRect(at,size)))
		{
			return;
		}
	}

	if (sf::IntRect(at,size).intersects(sf::IntRect(WIDTH,0,1,HEIGHT))) return;
	if (sf::IntRect(at,size).intersects(sf::IntRect(0,HEIGHT,WIDTH,1))) return;
	
	DungeonRoom* newRoom = new DungeonRoom();
	newRoom->type = m_CurrentRoomType;
	newRoom->space = sf::IntRect( at, size);
	m_DungeonRooms.push_back(newRoom);
	
	RefreshDungeon();
}

void RogueWindow::RemoveRoomAt( sf::Vector2i at)
{
	for (int i=0;i<m_DungeonRooms.size();i++)
	{
		if (m_DungeonRooms[i]->space.contains(at))
		{
			m_DungeonRooms.erase(m_DungeonRooms.begin()+i);
			break;
		}
	}
	RefreshDungeon();
}

void RogueWindow::DrawCorridor(sf::IntRect startroom, sf::IntRect endroom)
{
	sf::Vector2i startpoint = sf::Vector2i( startroom.left+1+DRand(startroom.width-2,startroom.left),startroom.top+1+DRand(startroom.height-2,startroom.top) );
	sf::Vector2i endpoint = sf::Vector2i( endroom.left+1+DRand(endroom.width-2,startroom.left),endroom.top+1+DRand(endroom.height-2,startroom.top) );
	
	sf::Vector2i point = sf::Vector2i(startpoint);
	
	int angle = DRand(3,endroom.left);
	if (angle == 0) // H then V
	{
		while (point.x != endpoint.x)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(point.x,point.y) ];
			thisrb->NewChar('#');
			
			if (point.x < endpoint.x)			point.x += 1;
			else if (point.x > endpoint.x)		point.x -= 1;
			else break;
		}
		while (point.y != endpoint.y)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(point.x,point.y) ];
			thisrb->NewChar('#');
			
			if (point.y < endpoint.y)			point.y += 1;
			else if (point.y > endpoint.y)		point.y -= 1;
			else break;
		}
	}
	else if (angle == 1) // V then H
	{
		while (point.y != endpoint.y)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(point.x,point.y) ];
			thisrb->NewChar('#');
			
			if (point.y < endpoint.y)			point.y += 1;
			else if (point.y > endpoint.y)		point.y -= 1;
			else break;
		}
		while (point.x != endpoint.x)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(point.x,point.y) ];
			thisrb->NewChar('#');
			
			if (point.x < endpoint.x)			point.x += 1;
			else if (point.x > endpoint.x)		point.x -= 1;
			else break;
		}
	}
	else // Half-H then V then Half-H
	{
		int halfway = abs(point.x - endpoint.x)/2;
		int gone = 0;
		
		while (point.x != endpoint.x || gone >= halfway)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(point.x,point.y) ];
			thisrb->NewChar('#');
			
			if (point.x < endpoint.x)			point.x += 1;
			else if (point.x > endpoint.x)		point.x -= 1;
			else break;
			
			gone++;
		}
		while (point.y != endpoint.y)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(point.x,point.y) ];
			thisrb->NewChar('#');
			
			if (point.y < endpoint.y)			point.y += 1;
			else if (point.y > endpoint.y)		point.y -= 1;
			else break;
		}
		while (point.x != endpoint.x)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(point.x,point.y) ];
			thisrb->NewChar('#');
			
			if (point.x < endpoint.x)			point.x += 1;
			else if (point.x > endpoint.x)		point.x -= 1;
			else break;
		}
	}
	
}

void RogueWindow::RefreshDungeon()
{
	std::string title = "";
	
	for (int i=0;i<WIDTH;i++)
	{
		for (int j=0;j<HEIGHT;j++)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(i,j) ];
			if (thisrb->m_ButtonChar != ' ')	thisrb->NewChar(' ');
		}
	}
	
	
	
	if (m_DungeonRooms.size() >= 2)
	{
		for (int i=1;i<m_DungeonRooms.size();i++)
		{
			DrawCorridor(m_DungeonRooms[0]->space,m_DungeonRooms[i]->space);
		}
	}
	
	if (m_DungeonRooms.size() >= 4)
	{
		DrawCorridor(m_DungeonRooms[2]->space,m_DungeonRooms[3]->space);
	}
	
	for (int i=0;i<m_DungeonRooms.size();i++)
	{
		
		if (m_DungeonRooms[i]->type != PLAYERBASE)
		{
			title += RoomTypeToChar(m_DungeonRooms[i]->type);
		}
		
		for (int y=0;y<m_DungeonRooms[i]->space.height;y++)
		{
			for (int x=0;x<m_DungeonRooms[i]->space.width;x++)
			{
				if (y==0 || y==m_DungeonRooms[i]->space.height-1)
				{
					RogueButton* thisrb = m_RogueButtons[ cord(
					m_DungeonRooms[i]->space.left+x,
					m_DungeonRooms[i]->space.top+y) ];
					thisrb->NewChar('-');
				}
				else if (x==0 || x==m_DungeonRooms[i]->space.width-1)
				{
					RogueButton* thisrb = m_RogueButtons[ cord(
					m_DungeonRooms[i]->space.left+x,
					m_DungeonRooms[i]->space.top+y) ];
					thisrb->NewChar('|');
				}
				else if ( x == m_DungeonRooms[i]->space.width/2 && y == m_DungeonRooms[i]->space.height/2)
				{
					RogueButton* thisrb = m_RogueButtons[ cord(
					m_DungeonRooms[i]->space.left+x,
					m_DungeonRooms[i]->space.top+y) ];
					thisrb->NewChar(RoomTypeToChar(m_DungeonRooms[i]->type));
				}
				else if ( m_DungeonRooms[i]->type == PING && x == 2+m_DungeonRooms[i]->space.width/2 && y == m_DungeonRooms[i]->space.height/2)
				{
					RogueButton* thisrb = m_RogueButtons[ cord(
					m_DungeonRooms[i]->space.left+x,
					m_DungeonRooms[i]->space.top+y) ];
					thisrb->NewChar('>');
				}
				else if ( m_DungeonRooms[i]->type == PING && x == -2+m_DungeonRooms[i]->space.width/2 && y == m_DungeonRooms[i]->space.height/2)
				{
					RogueButton* thisrb = m_RogueButtons[ cord(
					m_DungeonRooms[i]->space.left+x,
					m_DungeonRooms[i]->space.top+y) ];
					thisrb->NewChar('<');
				}
				else if ( 
					( m_DungeonRooms[i]->type == ROCK || m_DungeonRooms[i]->type == BPAPER || m_DungeonRooms[i]->type == XSCISSORS )
					&& 
					DRand(10,12*m_DungeonRooms[i]->space.left+15*x+34*m_DungeonRooms[i]->space.top+y) == 1 )
				{
					RogueButton* thisrb = m_RogueButtons[ cord(
					m_DungeonRooms[i]->space.left+x,
					m_DungeonRooms[i]->space.top+y) ];
					thisrb->NewChar(RoomTypeToChar(m_DungeonRooms[i]->type));
				}
				else
				{
					RogueButton* thisrb = m_RogueButtons[ cord(
					m_DungeonRooms[i]->space.left+x,
					m_DungeonRooms[i]->space.top+y) ];
					thisrb->NewChar('.');
				}
			}
		}
		
		
	/*
		
		// left wall
		for (int x=0;x<m_DungeonRooms[i]->space.height;x++)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(
				m_DungeonRooms[i]->space.left,
				m_DungeonRooms[i]->space.top+x) ];
			thisrb->NewChar('|');
		}
		// right wall
		for (int x=0;x<m_DungeonRooms[i]->space.height;x++)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(
				m_DungeonRooms[i]->space.left+m_DungeonRooms[i]->space.width-1,
				m_DungeonRooms[i]->space.top+x) ];
			thisrb->NewChar('|');
		}
		// top wall
		for (int x=0;x<m_DungeonRooms[i]->space.width;x++)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(
				m_DungeonRooms[i]->space.left+x,
				m_DungeonRooms[i]->space.top) ];
			thisrb->NewChar('-');
		}
		// bottom wall
		for (int x=0;x<m_DungeonRooms[i]->space.width;x++)
		{
			RogueButton* thisrb = m_RogueButtons[ cord(
				m_DungeonRooms[i]->space.left+x,
				m_DungeonRooms[i]->space.top+m_DungeonRooms[i]->space.height-1) ];
			thisrb->NewChar('-');
		}
		
		//centre
		RogueButton* thisrb = m_RogueButtons[ cord(
			m_DungeonRooms[i]->space.left+(m_DungeonRooms[i]->space.width/2),
			m_DungeonRooms[i]->space.top+(m_DungeonRooms[i]->space.height/2)) ];
		thisrb->NewChar(RoomTypeToChar(m_DungeonRooms[i]->type));
		
		*/
	
		
	}
	m_Description->SetText(title);
}

Object* RogueWindow::GetCurrentObject()
{
	Object* bObj = new Object();
	for (int i=0;i<m_DungeonRooms.size();i++)
	{
		bObj->m_Rooms.push_back(new Room(m_DungeonRooms[i]->type));
	}
	
	bObj->m_Name = m_Description->GetText();
	bObj->m_KnownEnemies = glob->world->GetHumanPlayer()->m_KnownEnemies;
	return bObj;
}

void RogueWindow::Clear()
{
	for (int i=0;i<m_DungeonRooms.size();i++)
	{
		delete m_DungeonRooms[i];
	}
	m_DungeonRooms.clear();
	
	RefreshAvailableRooms();
	RefreshDungeon();
}

void RogueWindow::Show(bool show)
{
	Clear();
	m_Window->Show(show);
	if (show) 	m_Window->GrabFocus();
}

void RogueWindow::Inventory()
{
	//TODO: check if valid - one @
	
	if (m_TargetButton == NULL)
	{
		m_TargetButton = m_CreateBar->AddNewButton();
	}
	
	m_TargetButton->m_Obj = GetCurrentObject();
	
	m_TargetButton->m_Button->SetLabel(m_Description->GetText());
	m_TargetButton = m_CreateBar->AddNewButton();
	Clear();
	Show(false);
}

void RogueWindow::Delete()
{
	Clear();
	Show(false);
}

void RogueWindow::Spawn()
{
	Object* newObj = GetCurrentObject();
	
	std::stringstream ss;
	ss << rand()%999;
	newObj->m_Name += ss.str();
	
	std::cout << newObj->m_Name << std::endl;
	
	newObj->m_KnownEnemies = glob->world->GetHumanPlayer()->m_KnownEnemies;
	
	glob->event->CreateNewObject( newObj, m_Description->GetText() );
}